class Wip():
    def print_it(self):
        print("itsa me mario")


def wet():
    print("beep boop")


def wat():
    "WEEE"
    print("testing, 1, 2, 3")


def wut(text, woop=False):
    """ Super docstring test

    Args:
        text (str): The text to print
        woop (boolean, optional): Default false
    """
    print(text)
    print(woop)


def _hidden():
    print("secret")
